String clazzTpl({package}) => """
import 'package:xanno/xanno.dart';
import 'package:flutter/material.dart';
import 'package:$package/widget/theme.dart';
import 'package:$package/generated/l10n.dart';
import 'package:$package/route/main.route.dart';
import 'package:$package/widget/observable.dart';
import 'package:$package/mixin/base_widget_mixin.dart';

class CounterState {
  int counter = 0;
}

var observableCounter = Observable(CounterState());

@GRoute(url: '/', title: 'main')
class MainPage extends BaseStatefulWidget {
  final String title;

  const MainPage({Key? key, required this.title}) : super(key: key);

  @override
  _MainPageState createState() => _MainPageState();
}

class _MainPageState extends BaseState<MainPage> with BaseStatefulMixin {
  @override
  void initState() {
    observableCounter.listen((state) {
      print(state.counter);
    });
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    super.build(context);
    return ThemeBuilder(
      builder: (context, theme) => Scaffold(
        backgroundColor: theme.body,
        appBar: AppBar(
          brightness: theme.type == ThemeType.light
              ? Brightness.light
              : Brightness.dark,
          backgroundColor: theme.body,
          title: Text(
            widget.title,
            style: TextStyle(
              fontSize: 30,
              fontWeight: FontWeight.bold,
              color: theme.title,
            ),
          ),
        ),
        body: Column(
          mainAxisAlignment: MainAxisAlignment.center,
          children: [
            Align(
              child: ObservableBuilder<CounterState>(
                create: observableCounter,
                memo: (state) => [state.counter],
                builder: (context, state) => Text(
                  '\${S.of(context).xanno} 👍 \${state.counter}',
                  style: TextStyle(
                    fontSize: 30,
                    fontWeight: FontWeight.bold,
                    color: theme.title,
                  ),
                ),
              ),
            ),
            GestureDetector(
              behavior: HitTestBehavior.opaque,
              onTap: () =>
                  Navigator.pushNamed(context, RouteConstant.AppDebugPage),
              child: Container(
                padding: EdgeInsets.all(10),
                decoration: ShapeDecoration(
                  color: theme.title.withOpacity(0.2),
                  shape: StadiumBorder(),
                ),
                child: Text(
                  S.of(context).desc,
                  style: TextStyle(color: theme.title),
                ),
              ),
            ),
          ],
        ),
        floatingActionButton: FloatingActionButton(
          onPressed: () {
            // counter++
            observableCounter.notify((state) => state.counter++);
            // 改变主题
            observableTheme.notify((theme) => theme.type =
                theme.type == ThemeType.light
                    ? ThemeType.dark
                    : ThemeType.light);
          },
          backgroundColor: theme.title,
          child: Icon(Icons.add, color: theme.body),
        ),
      ),
    );
  }
}
""";
